from django.shortcuts import render
from django.http import HttpResponse
from django.http import HttpResponseRedirect
from django.http import HttpResponseNotFound
from django.shortcuts import redirect
from django.utils import timezone
from django.views.decorators.cache import cache_control
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import smart_str

from .models import DownloadSite, Download, IsExpiredError, UploadFile
from . import presettings

import os

# Create your views here.
def downloadsite(request):
    if request.method == 'POST':
        sd = DownloadSite()
        sd.save()
        for down_file in UploadFile.objects.all():
            d = Download(site=sd, down_file=down_file)
            d.save()
        return HttpResponseRedirect('../site/' + sd.link_key +'/')
    return HttpResponse('Method Not Allowed', status=405)


def expired(key):
    expired_objects = Download.objects.filter(active=False)
    # check expired objects
    for obj in expired_objects:
        if key == obj.link_key:
            return obj


def active(key):
    active_objects = Download.objects.filter(active=True)
    # check active objects
    for obj in active_objects:
        if key == obj.link_key:
            return obj


def error(request, text=_(u'Sorry, your request is not available')):
    """returns the error page"""
    extra_context = {'message': text}
    template = 'not_avallible.html'
    return render(request, template, extra_context)


def site(request, offset):
    """process site requests"""
    obj = {'actives': [], 'expired': [], 'not_exist': []}

    sd = DownloadSite.objects.all().filter(link_key=offset).first()
    if sd is None:
        extra_context = {'message': 'This site does not exist'}
        template = 'not_avallible.html'
        return render(request, template, extra_context)

    # Test if keys are valid
    for download in Download.objects.all().filter(site=sd):
        key = download.link_key
        if active(key):
            obj['actives'].append(active(key))
        elif expired(key):
            obj['expired'].append(expired(key))
        else:
            obj['not_exist'].append(key)

    template = 'site.html'
    extra_context = {
        'basepath': presettings.DYNAMIC_LINK_URL,
        'downloads': obj
    }
    return render(request, template, extra_context)


def fetch(request, offset):
    """process link requests. make desissions for every single download link"""
    if active(offset):
        return provide(request, offset)
    elif expired(offset):
        return error(request, presettings.TEXT_REQUEST_IS_EXPIRED)
    else:
        return error(request, presettings.TEXT_REQUEST_DOES_NOT_EXIST)


@cache_control(private=True)
def provide(request, key):
    """
    Return a download without the rael path to the served file.
    The content will served by a stream.

    The file will read in byte code to a socket wich will be
    used in the response object. Headers in the response will
    set for the specific served content referable to its type.
    """
    stored_file_obj = Download.objects.get(link_key=key)
    try:
        # we use a model method to validate and deliver the path
        filepath = stored_file_obj.get_path()
    except IsExpiredError as e:
        return error(request, e.value)

    # make file path suitable for different installations
    delimiter = presettings.DYNAMIC_LINK_MEDIA.strip('/').strip('\\').split('/')[-1]
    # now we use the objects get_path() method to be sure
    # the object instance keep up to date.
    file_path = os.path.normpath(
        presettings.DYNAMIC_LINK_MEDIA + filepath.split(delimiter)[-1]
    )

    # get file parameters
    file_name = os.path.basename(file_path)
    file_size = os.path.getsize(file_path)

    if os.path.exists(file_path):
        try:
            with open(file_path, 'rb') as fr:
                response = HttpResponse(
                    fr.read(), content_type='application/force-download')
                response['Content-Disposition'] = 'attachment; filename=%s' \
                    % smart_str(file_name)
                response['X-Sendfile'] = smart_str(file_path)
                # response['X-Accel-Redirect'] = smart_str(file_path) # For ngnix
                response['Content-Length'] = file_size

                return response
        except IOError:
            stored_file_obj.active = False
            stored_file_obj.save()  # only raise the following once
            return HttpResponseNotFound(unicode(_(u'File not found!')))  # admin

    extra_context = {'message': 'Something went wrong...'}
    template = 'not_avallible.html'
    return render(request, template, extra_context)



from django.db import models

# Create your models here.
from django.db import models
from django.db import IntegrityError
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from . import presettings
from . import api

import os


class DownloadSite(models.Model):
    class Meta:
        verbose_name = 'Download Site'
    link_key = models.CharField(max_length=63, editable=False, unique=True)
    timestamp_creation = models.DateTimeField(
        auto_now=False,
        auto_now_add=True,
        editable=False,
        verbose_name=_(u'creation time'),
    )

    def save(self, * args, ** kwargs):
        """Perform custom methods before saving."""
        # If not available set a link key before saving
        if not self.link_key:
            self.link_key = api.gen_key()

        # call the real save method
        try:
            super(DownloadSite, self).save(*args, ** kwargs)
        except IntegrityError:
            # enforce a unique key (with a longer string)
            import time
            key = str(time.time()).replace('.', '')
            self.link_key = key + api.gen_key()
            # call the save method again
            super(DownloadSite, self).save(*args, ** kwargs)

    def __str__(self):
        return self.link_key


class UploadFile(models.Model):
    class Meta:
        verbose_name = 'Upload File'
    display_name = models.CharField(max_length=63, blank=False, null=False, unique=True, verbose_name=_('Display Name'))
    file = models.FileField(
        upload_to=presettings.DYNAMIC_LINK_UPLOAD_TO,
        help_text=_(u'Select the file to upload'),
        verbose_name=_(u'file'),
        max_length=255,
    )

    def __str__(self):
        return self.display_name


class Download(models.Model):
    active = models.BooleanField(default=True, verbose_name=_(u'is active'))
    link_key = models.CharField(max_length=63, editable=False, unique=True)
    timestamp_creation = models.DateTimeField(
        auto_now=False,
        auto_now_add=True,
        editable=False,
        verbose_name=_(u'creation time'),
    )
    site = models.ForeignKey(DownloadSite, models.CASCADE, related_name='downloads', blank=True, null=True)
    down_file = models.ForeignKey(UploadFile, models.CASCADE, related_name='downloads', blank=True, null=True)

    def get_filename(self):
        return os.path.basename(self.down_file.file.path)

    def get_path(self):
        """
        if active it returns the full path of stored file to serve.
        if clicked it returns None
        """
        if self.active:
            self.active = False
            self.save()
            return self.down_file.file.path
        return None

    def save(self, * args, ** kwargs):
        """Perform custom methods before saving."""
        # If link not created, generate a link key before saving
        if not self.link_key:
            self.link_key = api.gen_key()

        # call the real save method
        try:
            super(Download, self).save(*args, ** kwargs)
        except IntegrityError:
            # enforce a unique key (with a longer string)
            import time
            key = str(time.time()).replace('.', '')
            self.link_key = key + api.gen_key()
            # call the save method again
            super(Download, self).save(*args, ** kwargs)

    def __unicode__(self):
        if self.down_file is not None:
            return '%s: %s, %s' % (
                str(_(u'Filename')),
                self.down_file.display_name,
                self.get_filename()
            )
        else:
            return '%s: %s' % (
                str(_(u'Filename')),
                self.get_filename()
            )

    def __str__(self):
        if self.down_file is not None:
            return 'Name: %s  Key: %s  Path: %s' % (self.down_file.display_name, self.link_key, self.down_file.file.path)
        else:
            return self.link_key


class IsExpiredError(Exception):
    """Error class for expired link objects"""

    def __init__(self, value=''):
        self.value = presettings.TEXT_REQUEST_IS_EXPIRED + value

    def __str__(self):
        return repr(self.value)


from django.conf.urls import url

from . import views

app_name = 'OneTimeLink'

urlpatterns = [
    url(r'^site/([\w-]*)/$', views.site, name='site'),
    url(r'^link/(\w{31})/.*$', views.fetch, name='fetch'),
    url(r'^downloadsite/$', views.downloadsite, name='downloadsite'),
]



#!/usr/bin/python
# -*- coding:utf-8 -*-
# This Python file uses the following encoding: utf-8
from setuptools import setup, find_packages
import os
import sys


# Get description from Readme file
readme_file = os.path.join(os.path.dirname(__file__), 'README.rst')
long_description = open(readme_file).read()

# Build a list with requirements of the app
REQUIRES = []
try:
    import django
except ImportError:
    # Because of the strange update behavior of "pip --upgrade package_name"
    # set requierment only if django not avallible.
    REQUIRES.append('django')

if sys.version_info < (3, 6):
   REQUIRES.append('python >= 3.6')


setup(name='django-onetimelink',
        version=1.3,
        description='A django one-time file streaming application',
        long_description=long_description,
        author='FENG Hao',
        author_email='hiroshifuu@outlook.com',
        url='https://github.com/HiroshiFuu/django-onetimelink',
        download_url='https://pypi.python.org/pypi/django-onetimelink',
        license='BSD',
        packages=find_packages(exclude=['example', ]),
        include_package_data=True,
        keywords="django one-time file link serve",
        classifiers=[
              'Development Status :: 5 - Production/Stable',
              'Framework :: Django :: 2.0',
              'Framework :: Django :: 3.0',
              'License :: OSI Approved :: BSD License',
              'Operating System :: OS Independent',
              'Programming Language :: Python :: 3 :: Only',
              'Environment :: Console',
              'Natural Language :: English',
              'Intended Audience :: Developers',
              'Topic :: Internet',
              ],
        install_requires=REQUIRES,
        zip_safe=False,
        )



from . import presettings

import random


def file_link_url(request, linkobject):
    """returns the access url of the of the dynamicLink object"""
    return '%s://%s%s/link/%s/%s' % (
        presettings.DYNAMIC_LINK_SCHEMA_PROTO,
        request.META.get('HTTP_HOST'),
        presettings.DYNAMIC_LINK_URL,
        linkobject.link_key,
        linkobject.get_filename()
    )


def site_link_url(request, siteobj):
    """returns a site urls form already given keys"""
    return '%s://%s%s/site/%s' % (
        presettings.DYNAMIC_LINK_SCHEMA_PROTO,
        request.META.get('HTTP_HOST'),
        presettings.DYNAMIC_LINK_URL,
        siteobj.link_key
    )


def gen_key():
    """
    function for generating random keys
    """
    key = ''
    chars = presettings.KEY_POLL
    key_length = presettings.KEY_LENGTH
    for y in range(key_length):
        key += chars[random.randint(0, len(chars) - 1)]
    return key
